import 'package:dart_telegram_bot/tgapi/entities/message.dart';

class BotCommand {
  static final _commandRegex = RegExp(r'^(?:\/|!)(\w+)(?:@(\w+(?:bot)))?(?:\s(.+))?$', caseSensitive: false);

  String command;
  String botUsername;
  List<String> args = [];

  BotCommand(this.command, this.botUsername, {this.args});

  bool isToBot(String targetBotUsername) {
    return botUsername == null || targetBotUsername.toLowerCase() == botUsername.toLowerCase();
  }

  bool matchesCommand(String command, {String targetBotUsername}) {
    if (targetBotUsername != null) {
      if (!isToBot(targetBotUsername)) {
        return false;
      }
    }
    return command.toLowerCase() == this.command.toLowerCase();
  }

  factory BotCommand.fromMessage(Message message) {
    var match = _commandRegex.firstMatch(message.text);
    if (match == null) return null;

    var command = match.group(1);
    var username = match.group(2);
    var foundArgs = match.group(3);

    if (foundArgs != null) {
      return BotCommand(command, username, args: foundArgs.split(' '));
    }

    return BotCommand(command, username);
  }
}

